@@ -5,10 +5,26 @@
 ##
 
 #   parameters
-ssleay="$1"
-type="$2"
-crt="$3"
-key="$4"
+make="$1"
+mflags="$2"
+ssleay="$3"
+support="$4"
+type="$5"
+crt="$6"
+key="$7"
+
+#   we can operate only inside the Apache 1.3 source
+#   tree and only when mod_ssl+SSLeay is actually configured.
+if [ ! -f "../README.configure" ]; then
+    echo "mkcert.sh:Error: Cannot operate outside the Apache 1.3 source tree." 1>&2
+    echo "mkcert.sh:Hint:  You have to stay inside apache_1.3.x/src." 1>&2
+    exit 1
+fi
+if [ ".$ssleay" = . ]; then
+    echo "mkcert.sh:Error: mod_ssl/SSLeay has to be configured before using this utility." 1>&2
+    echo "mkcert.sh:Hint:  Configure mod_ssl with --enable-module=ssl in APACI, first." 1>&2
+    exit 1
+fi
 
 #   configuration
 #   WE ARE CALLED FROM THE PARENT DIR!
@@ -32,9 +48,14 @@ case $TERM in
         ;;
 esac
 
+#   display header
+echo "${T_MD}SSL Certificate Generation Utility${T_ME} (mkcert.sh)"
+echo "Copyright (c) 1998 Ralf S. Engelschall, All Rights Reserved."
+
 #   find some random files
 #   (do not use /dev/random here, because this device 
 #   doesn't work as expected on all platforms)
+echo " + finding random files on your platform"
 randfiles=''
 for file in /var/log/messages /var/adm/messages \
             /kernel /vmunix /vmlinuz \
@@ -48,6 +69,24 @@ for file in /var/log/messages /var/adm/messages \
     fi
 done
 
+#   on-demand compile the ca-fix only
+case $type in
+    test|custom)
+       cd $support
+       if [ ! -f ca-fix ]; then
+           echo " + building ca-fix auxiliary tool"
+           $make $mflags ca-fix >/dev/null 2>&1
+           if [ $? -ne 0 ]; then
+               $make $mflags ca-fix
+               echo "**FAILED"
+               exit 1
+           fi
+       fi
+       cd ..
+       cafix="$support/ca-fix"
+       ;;
+esac
+
 #   processing
 case $type in
 
@@ -135,19 +174,64 @@ EOT
         echo "______________________________________________________________________"
         echo ""
         echo "${T_MD}STEP 3: Generating X.509 certificate signed by Snake Oil CA [server.crt]${T_ME}"
+        echo dummy | awk '{ printf("%s", prompt); }' "prompt=Certificate Version (1 or 3) [3]:"
+        read certversion
+        if [ ".$certversion" = .3 -o ".$certversion" = . ]; then
+            certversion=3
+        else
+            certversion=1
+        fi
         if [ ! -f .mkcert.serial ]; then
             echo '01' >.mkcert.serial
         fi
         $ssleay x509 -days 365 \
                      -CAserial .mkcert.serial \
-                     -CA $sslcrtdir/snakeoil.crt \
-                     -CAkey $sslkeydir/snakeoil.key \
+                     -CA $sslcrtdir/snakeoil-ca.crt \
+                     -CAkey $sslkeydir/snakeoil-ca.key \
                      -in $sslcsrdir/server.csr -req \
                      -out $sslcrtdir/server.crt
         if [ $? -ne 0 ]; then
             echo "mkcert.sh:Error: Failed to generate X.509 certificate" 1>&2
             exit 1
         fi
+        if [ ".$certversion" = .3 ]; then
+            echo "Converting X.509 v1 to v3 certificate"
+            $cafix -nscertype 0x40 \
+                   -pathlen 0 \
+                   -nobscrit \
+                   -nosign \
+                   -in    $sslcrtdir/server.crt \
+                   -inkey $sslkeydir/server.key \
+                   -out   $sslcrtdir/server.crt.fixed
+            if [ $? -ne 0 ]; then
+                echo "mkcert.sh:Error: Failed to fixup X.509 certificate contents" 1>&2
+                exit 1
+            fi
+            cp $sslcrtdir/server.crt.fixed $sslcrtdir/server.crt
+            rm -f $sslcrtdir/server.crt.fixed
+            $cafix -inkey $sslkeydir/snakeoil-ca.key \
+                   -in    $sslcrtdir/server.crt \
+                   -out   $sslcrtdir/server.crt.fixed
+            if [ $? -ne 0 ]; then
+                echo "mkcert.sh:Error: Failed to fixup X.509 certificate signature" 1>&2
+                exit 1
+            fi
+            cp $sslcrtdir/server.crt.fixed $sslcrtdir/server.crt
+            rm -f $sslcrtdir/server.crt.fixed
+        fi
+        echo "Verify: matching certificate & key modulus"
+        modcrt=`$ssleay x509 -noout -modulus -in $sslcrtdir/server.crt`
+        modkey=`$ssleay rsa  -noout -modulus -in $sslkeydir/server.key`
+        if [ ".$modcrt" != ".$modkey" ]; then
+            echo "mkcert.sh:Error: Failed to verify modulus on resulting X.509 certificate" 1>&2
+            exit 1
+        fi
+        echo "Verify: matching certificate signature"
+        $ssleay verify -CAfile $sslcrtdir/snakeoil-ca.crt $sslcrtdir/server.crt
+        if [ $? -ne 0 ]; then
+            echo "mkcert.sh:Error: Failed to verify signature on resulting X.509 certificate" 1>&2
+            exit 1
+        fi
         echo "______________________________________________________________________"
         echo ""
         echo "${T_MD}STEP 4: Enrypting RSA private key with a pass phrase for security [server.key]${T_ME}"
@@ -190,7 +274,7 @@ EOT
         echo "${T_MD}Generating custom certificate signed by own CA [CUSTOM]${T_ME}"
         echo "______________________________________________________________________"
         echo ""
-        echo "${T_MD}STEP 1: Generating RSA private key for CA (1024 bit)${T_ME}"
+        echo "${T_MD}STEP 1: Generating RSA private key for CA (1024 bit) [ca.key]${T_ME}"
         if [ ! -f $HOME/.rnd ]; then
             touch $HOME/.rnd
         fi
@@ -208,7 +292,7 @@ EOT
         fi
         echo "______________________________________________________________________"
         echo ""
-        echo "${T_MD}STEP 2: Generating X.509 certificate signing request for CA${T_ME}"
+        echo "${T_MD}STEP 2: Generating X.509 certificate signing request for CA [ca.csr]${T_ME}"
         cat >.mkcert.cfg <<EOT
 [ req ]
 default_bits                    = 1024
@@ -244,19 +328,53 @@ EOT
         rm -f .mkcert.cfg
         echo "______________________________________________________________________"
         echo ""
-        echo "${T_MD}STEP 3: Generating X.509 certificate for CA signed by itself${T_ME}"
-        $ssleay x509 -req \
-                     -days 365 \
+        echo "${T_MD}STEP 3: Generating X.509 certificate for CA signed by itself [ca.crt]${T_ME}"
+        echo dummy | awk '{ printf("%s", prompt); }' "prompt=Certificate Version (1 or 3) [3]:"
+        read certversion
+        if [ ".$certversion" = .3 -o ".$certversion" = . ]; then
+            certversion=3
+        else
+            certversion=1
+        fi
+        $ssleay x509 -days 365 \
                      -signkey $sslkeydir/ca.key \
-                     -in  $sslcsrdir/ca.csr \
-                     -out $sslcrtdir/ca.crt
+                     -in      $sslcsrdir/ca.csr -req \
+                     -out     $sslcrtdir/ca.crt
         if [ $? -ne 0 ]; then
             echo "mkcert.sh:Error: Failed to generate self-signed CA certificate" 1>&2
             exit 1
         fi
+        if [ ".$certversion" = .3 ]; then
+            $cafix -caset \
+                   -nscertype 0x07 \
+                   -pathlen 0 \
+                   -nobscrit \
+                   -in    $sslcrtdir/ca.crt \
+                   -inkey $sslkeydir/ca.key \
+                   -out   $sslcrtdir/ca.crt.fixed
+            if [ $? -ne 0 ]; then
+                echo "mkcert.sh:Error: Failed to fixup X.509 CA certificate" 1>&2
+                exit 1
+            fi
+            cp $sslcrtdir/ca.crt.fixed $sslcrtdir/ca.crt
+            rm -f $sslcrtdir/ca.crt.fixed
+        fi
+        echo "Verify: matching certificate & key modulus"
+        modcrt=`$ssleay x509 -noout -modulus -in $sslcrtdir/ca.crt`
+        modkey=`$ssleay rsa  -noout -modulus -in $sslkeydir/ca.key`
+        if [ ".$modcrt" != ".$modkey" ]; then
+            echo "mkcert.sh:Error: Failed to verify modulus on resulting X.509 certificate" 1>&2
+            exit 1
+        fi
+        echo "Verify: matching certificate signature"
+        $ssleay verify $sslcrtdir/ca.crt
+        if [ $? -ne 0 ]; then
+            echo "mkcert.sh:Error: Failed to verify signature on resulting X.509 certificate" 1>&2
+            exit 1
+        fi
         echo "______________________________________________________________________"
         echo ""
-        echo "${T_MD}STEP 4: Generating RSA private key for SERVER (1024 bit)${T_ME}"
+        echo "${T_MD}STEP 4: Generating RSA private key for SERVER (1024 bit) [server.key]${T_ME}"
         if [ ! -f $HOME/.rnd ]; then
             touch $HOME/.rnd
         fi
@@ -274,7 +392,7 @@ EOT
         fi
         echo "______________________________________________________________________"
         echo ""
-        echo "${T_MD}STEP 5: Generating X.509 certificate signing request for SERVER${T_ME}"
+        echo "${T_MD}STEP 5: Generating X.509 certificate signing request for SERVER [server.csr]${T_ME}"
         cat >.mkcert.cfg <<EOT
 [ req ]
 default_bits                    = 1024
@@ -310,23 +428,67 @@ EOT
         rm -f .mkcert.cfg
         echo "______________________________________________________________________"
         echo ""
-        echo "${T_MD}STEP 6: Generating X.509 certificate signed by own CA${T_ME}"
+        echo "${T_MD}STEP 6: Generating X.509 certificate signed by own CA [server.crt]${T_ME}"
+        echo dummy | awk '{ printf("%s", prompt); }' "prompt=Certificate Version (1 or 3) [3]:"
+        read certversion
+        if [ ".$certversion" = .3 -o ".$certversion" = . ]; then
+            certversion=3
+        else
+            certversion=1
+        fi
         if [ ! -f .mkcert.serial ]; then
             echo '01' >.mkcert.serial
         fi
         $ssleay x509 -days 365 \
                      -CAserial .mkcert.serial \
-                     -CA $sslcrtdir/ca.crt \
+                     -CA    $sslcrtdir/ca.crt \
                      -CAkey $sslkeydir/ca.key \
-                     -in $sslcsrdir/server.csr -req \
-                     -out $sslcrtdir/server.crt
+                     -in    $sslcsrdir/server.csr -req \
+                     -out   $sslcrtdir/server.crt
         if [ $? -ne 0 ]; then
             echo "mkcert.sh:Error: Failed to generate X.509 certificate" 1>&2
             exit 1
         fi
+        if [ ".$certversion" = .3 ]; then
+            $cafix -nscertype 0x40 \
+                   -pathlen 0 \
+                   -nobscrit \
+                   -nosign \
+                   -in    $sslcrtdir/server.crt \
+                   -inkey $sslkeydir/server.key \
+                   -out   $sslcrtdir/server.crt.fixed
+            if [ $? -ne 0 ]; then
+                echo "mkcert.sh:Error: Failed to fixup X.509 certificate contents" 1>&2
+                exit 1
+            fi
+            cp $sslcrtdir/server.crt.fixed $sslcrtdir/server.crt
+            rm -f $sslcrtdir/server.crt.fixed
+            $cafix -inkey $sslkeydir/ca.key \
+                   -in    $sslcrtdir/server.crt \
+                   -out   $sslcrtdir/server.crt.fixed
+            if [ $? -ne 0 ]; then
+                echo "mkcert.sh:Error: Failed to fixup X.509 certificate signature" 1>&2
+                exit 1
+            fi
+            cp $sslcrtdir/server.crt.fixed $sslcrtdir/server.crt
+            rm -f $sslcrtdir/server.crt.fixed
+        fi
+        echo "Verify: matching certificate & key modulus"
+        modcrt=`$ssleay x509 -noout -modulus -in $sslcrtdir/server.crt`
+        modkey=`$ssleay rsa  -noout -modulus -in $sslkeydir/server.key`
+        if [ ".$modcrt" != ".$modkey" ]; then
+            echo "mkcert.sh:Error: Failed to verify modulus on resulting X.509 certificate" 1>&2
+            exit 1
+        fi
+        echo "Verify: matching certificate signature"
+        $ssleay verify -CAfile $sslcrtdir/ca.crt $sslcrtdir/server.crt
+        if [ $? -ne 0 ]; then
+            echo "mkcert.sh:Error: Failed to verify signature on resulting X.509 certificate" 1>&2
+            exit 1
+        fi
         echo "______________________________________________________________________"
         echo ""
-        echo "${T_MD}STEP 7: Enrypting RSA private key of CA with a pass phrase for security${T_ME}"
+        echo "${T_MD}STEP 7: Enrypting RSA private key of CA with a pass phrase for security [ca.key]${T_ME}"
         $ssleay rsa -des3 -in $sslkeydir/ca.key -out $sslkeydir/ca.key.crypt
         if [ $? -ne 0 ]; then
             echo "mkcert.sh:Error: Failed to encrypt RSA private key" 1>&2
@@ -336,7 +498,7 @@ EOT
         rm -f $sslkeydir/ca.key.crypt
         echo "______________________________________________________________________"
         echo ""
-        echo "${T_MD}STEP 8: Enrypting RSA private key of SERVER with a pass phrase for security${T_ME}"
+        echo "${T_MD}STEP 8: Enrypting RSA private key of SERVER with a pass phrase for security [server.key]${T_ME}"
         $ssleay rsa -des3 -in $sslkeydir/server.key -out $sslkeydir/server.key.crypt
         if [ $? -ne 0 ]; then
             echo "mkcert.sh:Error: Failed to encrypt RSA private key" 1>&2
