@@ -1013,7 +1013,7 @@ static int ssl_hook_Auth(request_rec *r)
      */
     ap_snprintf(b1, sizeof(b1), "%s:password", 
                 r->connection->client->szClientX509);
-    util_uuencode(b2, b1);
+    util_uuencode(b2, b1, FALSE);
     ap_snprintf(b1, sizeof(b1), "Basic %s", b2);
     ap_table_set(r->headers_in, "Authorization", b1);
 
@@ -1169,8 +1169,10 @@ static int ssl_callback_SSLVerify(int ok, X509_STORE_CTX *ctx)
 
     SSLConfigRec *pConfig = mySrvConfig(g_pCurrentConnection->server);
 
-    if (   error == X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY 
-        || error == X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE) {
+    if (   error == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT
+        || error == X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN
+        || error == X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY 
+        || error == X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE  ) {
         char *s;
 
         if (pConfig->nVerifyClient == VERIFY_OPTIONAL_NO_CA) {
@@ -1338,9 +1340,9 @@ static int ssl_CacheLauncher(void *cmd, child_info *pInfo)
 
 static void ssl_SendSessionToServer(SSL_SESSION *pSession)
 {
-    uchar buf[MAX_SESSION_DER];
+    UCHAR buf[MAX_SESSION_DER];
     Cache *p;
-    uchar *t;
+    UCHAR *t;
     int nLength;
     SSLConfigRec *pConfig;
     time_t tExpiresAt;
@@ -1368,11 +1370,11 @@ static void ssl_SendSessionToServer(SSL_SESSION *pSession)
     return;
 }
 
-static SSL_SESSION *ssl_GetSessionFromServer(uchar *aucSessionID, int nLength)
+static SSL_SESSION *ssl_GetSessionFromServer(UCHAR *aucSessionID, int nLength)
 {
     SSL_SESSION *pSession;
     Cache *p;
-    uchar *t, *t2;
+    UCHAR *t, *t2;
     int nData;
     time_t tExpiresAt;
 
@@ -1472,23 +1474,37 @@ static void util_strupper(char *s)
 static const char util_uuencode_six2pr[64+1] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
-static void util_uuencode(char *szTo, const char *szFrom)
+static void util_uuencode(char *szTo, const char *szFrom, BOOL bPad) 
+{
+    util_uuencode_binary((unsigned char *)szTo, 
+                         (const unsigned char *)szFrom,
+                         strlen(szFrom), bPad);
+}
+
+static void util_uuencode_binary(
+    unsigned char *szTo, const unsigned char *szFrom, int nLength, BOOL bPad)
 {
     const unsigned char *s;
+    int nPad = 0;
 
-    for (s = (const unsigned char *)szFrom; *s; s += 3) {
+    for (s = szFrom; nLength > 0; s += 3) {
         *szTo++ = util_uuencode_six2pr[s[0] >> 2];
         *szTo++ = util_uuencode_six2pr[(s[0] << 4 | s[1] >> 4) & 0x3f];
-        if (!s[0])
+        if (--nLength == 0) {
+            nPad = 2;
             break;
+        }
         *szTo++ = util_uuencode_six2pr[(s[1] << 2 | s[2] >> 6) & 0x3f];
-        if (!s[1])
+        if (--nLength == 0) {
+            nPad = 1;
             break;
+        }
         *szTo++ = util_uuencode_six2pr[s[2] & 0x3f];
-        if (!s[2])
-            break;
+        --nLength;
     }
-    *szTo++ = NUL;
+    while(bPad && nPad--)
+        *szTo++ = NUL;
+    *szTo = NUL;
     return;
 }
 
