@@ -83,12 +83,10 @@
  * Provide reasonable defines for some types
  */
 #ifndef BOOL
-#define BOOL bool
-typedef unsigned int bool;
+#define BOOL unsigned int
 #endif
 #ifndef UCHAR
-#define UCHAR uchar
-typedef unsigned char uchar;
+#define UCHAR unsigned char
 #endif
 
 /*
@@ -237,7 +235,7 @@ static void         ssl_int_SetupVerify(conn_rec *);
 static void         ssl_LaunchCache(server_rec *, pool *);
 static int          ssl_CacheLauncher(void *, child_info *);
 static void         ssl_SendSessionToServer(SSL_SESSION *);
-static SSL_SESSION *ssl_GetSessionFromServer(uchar *, int);
+static SSL_SESSION *ssl_GetSessionFromServer(UCHAR *, int);
 
 /*  Logfile Support  */
 static void         ssl_log_own(SSLConfigRec *, const char *, ...);
@@ -245,6 +243,7 @@ static void         ssl_log_ssleay2apache(server_rec *);
 
 /*  Utility Functions  */
 static void         util_strupper(char *);
-static void         util_uuencode(char *, const char *);
+static void         util_uuencode(char *, const char *, BOOL); 
+static void         util_uuencode_binary(unsigned char *, const unsigned char *, int, BOOL);
 
 #endif /* MOD_SSL_H */
